@@ -1,4 +1,4 @@
-/* Copyright (C) 2002, 2003 Thorsten Kukuk
+/* Copyright (C) 2002, 2003, 2004 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
@@ -479,7 +479,7 @@ write_user_data (user_t *data, int is_locked)
 	    }
           fchmod (newpf_fd, passwd_stat.st_mode);
           fchown (newpf_fd, passwd_stat.st_uid, passwd_stat.st_gid);
-	  if (copy_xattr (passwd_orig, passwd_tmp) > 0)
+	  if (copy_xattr (passwd_orig, passwd_tmp) != 0)
 	    {
 	      fclose (oldpf);
 	      close (newpf_fd);
@@ -621,12 +621,13 @@ write_user_data (user_t *data, int is_locked)
 	 something, too. But do this only, if there no error occured
 	 before.  */
       if (data->use_shadow && retval == 0 &&
-	  (data->todo == DO_CREATE || data->todo == DO_DELETE ||
-	   data->todo == DO_CREATE_SHADOW ||
+	  (data->todo == DO_CREATE || data->todo == DO_CREATE_SHADOW ||
+	   data->todo == DO_DELETE || data->todo == DO_DELETE_SHADOW ||
 	   data->new_name || data->newpassword || data->sp_changed))
         {
 	  const char *file_tmp = "/shadow.tmpXXXXXX";
-	  char *shadow_tmp = alloca (strlen (files_etc_dir) + strlen (file_tmp) + 1);
+	  char *shadow_tmp = alloca (strlen (files_etc_dir) +
+				     strlen (file_tmp) + 1);
 	  char *shadow_orig = alloca (strlen (files_etc_dir) + 8);
 	  char *shadow_old = alloca (strlen (files_etc_dir) + 12);
           struct stat shadow_stat;
@@ -682,7 +683,7 @@ write_user_data (user_t *data, int is_locked)
             }
           fchmod (newpf_fd, shadow_stat.st_mode);
           fchown (newpf_fd, shadow_stat.st_uid, shadow_stat.st_gid);
-	  if (copy_xattr (shadow_orig, shadow_tmp) > 0)
+	  if (copy_xattr (shadow_orig, shadow_tmp) != 0)
 	    {
 	      fclose (oldpf);
 	      close (newpf_fd);
@@ -728,7 +729,8 @@ Error while writing temporary shadow file: %m\n"));
 		      gotit = 1;
 		    }
 		}
-	      else if (data->todo == DO_DELETE)
+	      else if (data->todo == DO_DELETE ||
+		       data->todo == DO_DELETE_SHADOW)
 		{
 		  if (strcmp (data->pw.pw_name, sp->sp_namp) == 0)
 		    {
