@@ -39,24 +39,12 @@
 #include <security/pam_misc.h>
 
 #include "error_codes.h"
-#include "getdef.h"
 #include "public.h"
-#include "main.h"
+#include "logindefs.h"
 #include "read-files.h"
 
 #ifdef USE_LDAP
-#ifdef HAVE_LBER_H
-#include <lber.h>
-#endif
-#ifdef HAVE_LDAP_H
-#include <ldap.h>
-#endif
-#ifdef HAVE_LDAP_SSL_H
-#include <ldap_ssl.h>
-#endif
-
-#define _INCLUDED_FROM_LDAPCFN_C_
-#include "ldapfcn.h"
+#include "libldap.h"
 #endif
 
 #ifndef _
@@ -66,19 +54,6 @@
 #define DAY (24L*3600L)
 #define SCALE DAY
 
-/* Print the version information.  */
-void
-print_version (const char *program)
-{
-  fprintf (stdout, "%s (%s) %s\n", program, PACKAGE, VERSION);
-  fprintf (stdout, gettext ("\
-Copyright (C) %s Thorsten Kukuk.\n\
-This is free software; see the source for copying conditions.  There is NO\n\
-warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
-"), "2002, 2003");
-  /* fprintf (stdout, _("Written by %s.\n"), "Thorsten Kukuk"); */
-}
-
 static void
 print_usage (FILE *stream, const char *program)
 {
@@ -110,14 +85,6 @@ print_help (const char *program)
   fputs (_("Valid services for -r are: files, nis, nisplus, ldap\n"), stdout);
 }
 
-void
-print_error (const char *program)
-{
-  fprintf (stderr,
-           _("Try `%s --help' or `%s --usage' for more information.\n"),
-           program, program);
-}
-
 /* Print the time in a human readable format.  */
 static void
 print_date (time_t date)
@@ -139,43 +106,6 @@ print_date (time_t date)
 #endif
 }
 
-/* convert time_t into a readable date string.  */
-static char *
-date2str (time_t date)
-{
-  struct tm *tp;
-  char buf[12];
-
-  tp = gmtime (&date);
-#ifdef HAVE_STRFTIME
-  strftime (buf, sizeof (buf), "%Y-%m-%d", tp);
-#else
-  snprintf (buf, sizeof (buf), "%04d-%02d-%02d",
-	    tp->tm_year + 1900, tp->tm_mon + 1, tp->tm_mday);
-#endif
-  return strdup (buf);
-}
-
-/* convert a string to a time_t value.  */
-static long int
-str2date (const char *str)
-{
-  struct tm tp;
-  char *cp;
-  time_t result;
-
-  memset (&tp, 0, sizeof tp);
-  cp = strptime (str, "%Y-%m-%d", &tp);
-  if (!cp || *cp != '\0')
-    return -1;
-
-  result = mktime (&tp);
-  if (result == (time_t) -1)
-    return -1;
-
-  return (result + (DAY/2)) / DAY;
-}
-
 /* Print the current values of the expiration fields.  */
 static void
 print_shadow_info (user_t *data)
@@ -307,40 +237,6 @@ change_shadow_info (user_t *data)
   return 0;
 }
 
-static void
-init_environment (void)
-{
-  struct rlimit rlim;
-
-  /* Don't create a core file.  */
-  rlim.rlim_cur = rlim.rlim_max = 0;
-  setrlimit (RLIMIT_CORE, &rlim);
-
-  /* Set all limits to unlimited to avoid to run in any
-     problems later.  */
-  rlim.rlim_cur = rlim.rlim_max = RLIM_INFINITY;
-  setrlimit (RLIMIT_AS, &rlim);
-  setrlimit (RLIMIT_CPU, &rlim);
-  setrlimit (RLIMIT_DATA, &rlim);
-  setrlimit (RLIMIT_FSIZE, &rlim);
-  setrlimit (RLIMIT_NOFILE, &rlim);
-  setrlimit (RLIMIT_RSS, &rlim);
-  setrlimit (RLIMIT_STACK, &rlim);
-
-  /* Ignore all signals which can make trouble later.  */
-  signal (SIGALRM, SIG_IGN);
-  signal (SIGXFSZ, SIG_IGN);
-  signal (SIGHUP, SIG_IGN);
-  signal (SIGINT, SIG_IGN);
-  signal (SIGPIPE, SIG_IGN);
-  signal (SIGQUIT, SIG_IGN);
-  signal (SIGTERM, SIG_IGN);
-  signal (SIGTSTP, SIG_IGN);
-  signal (SIGTTOU, SIG_IGN);
-
-  umask (077);
-}
-
 int
 main (int argc, char *argv[])
 {
@@ -473,7 +369,7 @@ main (int argc, char *argv[])
           print_help (program);
           return 0;
         case 'v':
-          print_version (program);
+          print_version (program, "2003");
           return 0;
         case 'u':
           print_usage (stdout, program);
@@ -538,26 +434,62 @@ main (int argc, char *argv[])
       if (pw_data == NULL || pw_data->service == S_NONE)
 	{
 	  if (use_service)
-	    fprintf (stderr, _("%s: User %s is not known to service %s\n"),
+	    fprintf (stderr, _("%s: User `%s' is not known to service `%s'.\n"),
 		     program, arg_user, use_service);
 	  else
-	    fprintf (stderr, _("%s: Unknown user %s\n"), program,
+	    fprintf (stderr, _("%s: Unknown user `%s'.\n"), program,
 		     arg_user);
 	  return E_UNKNOWN_USER;
 	}
     }
 
-  /* Only root is allowed to change aging for local users. */
-  if (!l_flag && uid && (pw_data->service == S_LOCAL
-#ifdef USE_LDAP
-			 || (pw_data->service == S_LDAP && binddn == NULL)
-#endif
-			 ))
+  if (!l_flag)
     {
-      fprintf (stderr,
-	       _("Only an administrator is allowed to change aging information.\n"));
-      free_user_t (pw_data);
-      return E_NOPERM;
+      /* Only root is allowed to change aging for local users. */
+      if (uid && (pw_data->service == S_LOCAL
+#ifdef USE_LDAP
+		  || (pw_data->service == S_LDAP && binddn == NULL)
+#endif
+		  ))
+	{
+	  fprintf (stderr,
+		   _("Only an administrator is allowed to change aging information.\n"));
+	  free_user_t (pw_data);
+	  return E_NOPERM;
+	}
+
+      /* If no shadow entry exist for this account, check if we can
+	 create them.  */
+      if (!pw_data->use_shadow)
+	{
+	  char shadowfile[strlen (files_etc_dir) + 8];
+	  char *cp = stpcpy (shadowfile, files_etc_dir);
+	  strcpy (cp, "/shadow");
+
+	  if (access (shadowfile, F_OK) != 0)
+	    {
+	      fprintf (stderr,
+		       _("This system does not support shadow accounts.\n"));
+	      return E_MISSING;
+	    }
+	  else if (pw_data->service != S_LOCAL)
+	    {
+	      fprintf (stderr,
+		       _("This account does not have a shadow entry.\n"));
+	      return E_MISSING;
+	    }
+	  else
+	    {
+	      /* Initialize data with dummy values. */
+	      pw_data->sp.sp_lstchg = -1;
+	      pw_data->sp.sp_min = -1;
+	      pw_data->sp.sp_max = -1;
+	      pw_data->sp.sp_warn = -1;
+	      pw_data->sp.sp_inact = -1;
+	      pw_data->sp.sp_expire = -1;
+	      pw_data->sp.sp_flag = -1;
+	    }
+	}
     }
 
 #ifdef USE_LDAP
@@ -565,105 +497,35 @@ main (int argc, char *argv[])
     {
       /* A user tries to change data stored in a LDAP database and
 	 knows the Manager dn, now we need the password from him.  */
-      char prompt[130+strlen (binddn)], *cp;
       ldap_session_t *session = create_ldap_session (LDAP_PATH_CONF);
-      int rc, msgid;
-      struct timeval timeout;
-      LDAPMessage *result;
-#if defined(HAVE_LDAP_PARSE_RESULT) && defined(HAVE_LDAP_CONTROLS_FREE)
-      int parserc;
-#endif
-#ifdef HAVE_LDAP_CONTROLS_FREE
-      LDAPControl **controls;
-#endif
+      char *cp;
 
       if (session == NULL)
 	return E_FAILURE;
 
-      snprintf (prompt, sizeof (prompt), _("Enter LDAP Password:"));
-      cp = getpass (prompt);
+      cp = getpass (_("Enter LDAP Password:"));
 
       pw_data->binddn = strdup (binddn);
 
       if (open_ldap_session (session) != 0)
 	return E_FAILURE;
 
-      msgid = ldap_simple_bind (session->ld, binddn, cp);
-      if (msgid == -1)
-	{
-#if 0 /* XXX */
-	  fprintf (stderr, "ldap_simple_bind %s",
-		   ldap_err2string (ldap_get_lderrno (session->ld, 0, 0)));
-#endif
-	  _pam_overwrite (cp);
-	  return E_NOPERM;
-	}
+      if (ldap_authentication (session, NULL, binddn, cp) != 0)
+	return E_NOPERM;
 
-      timeout.tv_sec = 10;
-      timeout.tv_usec = 0;
-      rc = ldap_result (session->ld, msgid, FALSE, &timeout, &result);
-      if (rc == -1 || rc == 0)
-	{
-	  fprintf (stderr, "ldap_result %s\n",
-		   ldap_err2string (ldap_get_lderrno (session->ld, 0, 0)));
-	  _pam_overwrite (cp);
-	  return E_NOPERM;
-	}
-
-#if defined(HAVE_LDAP_PARSE_RESULT) && defined(HAVE_LDAP_CONTROLS_FREE)
-      controls = 0;
-      parserc =
-	ldap_parse_result (session->ld, result, &rc, 0, 0, 0, &controls, TRUE);
-      if (parserc != LDAP_SUCCESS)
-	{
-	  if (isatty (fileno (stderr)))
-	    fprintf (stderr, "ldap_parse_result %s\n",
-		     ldap_err2string (parserc));
-	  else
-	    syslog (LOG_ERR, "ldap_parse_result %s",
-		    ldap_err2string (parserc));
-	  _pam_overwrite (session->info->userpw);
-	  _pam_drop (session->info->userpw);
-	  return parserc;
-	}
-
-      if (controls != NULL)
-	{
-	  LDAPControl **ctlp;
-	  for (ctlp = controls; *ctlp != NULL; ctlp++)
-	    {
-	      if (!strcmp ((*ctlp)->ldctl_oid, LDAP_CONTROL_PWEXPIRING))
-		{
-		  char seconds[32];
-		  snprintf (seconds, sizeof seconds, "%.*s",
-			    (int) (*ctlp)->ldctl_value.bv_len,
-			    (*ctlp)->ldctl_value.bv_val);
-		  session->info->password_expiration_time = atol (seconds);
-		}
-	      else if (!strcmp ((*ctlp)->ldctl_oid, LDAP_CONTROL_PWEXPIRED))
-		{
-		  session->info->password_expired = 1;
-		  _pam_overwrite (session->info->userpw);
-		  _pam_drop (session->info->userpw);
-		  rc = LDAP_SUCCESS;
-		  /* That may be a lie, but we need to get to the acct_mgmt
-		   * step and force the change...
-		   */
-		}
-	    }
-	  ldap_controls_free (controls);
-	}
-#else
-      rc = ldap_result2error (session->ld, result, TRUE);
-#endif
+      close_ldap_session (session);
 
       pw_data->oldclearpwd = strdup (cp);
     }
   else
 #endif /* USE_LDAP */
-    /* misuse l_flag: We don't need to extra ask for a password with "-l".  */
-    if (do_authentication (program, caller_name, pw_data, !l_flag) != 0)
+    if (do_authentication (program, caller_name, pw_data) != 0)
       return E_NOPERM;
+  /* We don't need to extra ask for a password with "-l" and if the
+     password is stored in the local file.  */
+    else if (!l_flag && pw_data->service != S_LOCAL)
+      if (get_old_clear_password (pw_data) != 0)
+	return E_FAILURE;
 
   if (l_flag)
     {
@@ -803,16 +665,52 @@ main (int argc, char *argv[])
       return 0;
     }
   else
-    pw_data->sp_changed = TRUE;
+    {
+      pw_data->sp_changed = TRUE;
+      pw_data->todo = DO_MODIFY;
+    }
 
 #ifdef USE_LDAP
   if (binddn)
     pw_data->binddn = strdup (binddn);
 #endif
 
-  if (do_setpwnam (pw_data) != 0)
+  /* We have a shadow file, but this user does not have
+     a shadow entry. Create one.  */
+  if (!pw_data->use_shadow)
     {
-      fprintf (stderr,_( "Error while changing aging information.\n"));
+      int rc;
+
+      /* Backup original password and replace it with a "x"
+	 in local files. Report error*/
+      pw_data->todo = DO_MODIFY;
+      pw_data->sp.sp_pwdp = pw_data->pw.pw_passwd;
+      pw_data->newpassword = "x";
+      rc = write_user_data (pw_data, 0);
+      pw_data->newpassword = NULL;
+
+      if (rc != 0)
+	{
+	  fprintf (stderr,
+		   _("Error while converting to shadow account.\n"));
+	  free_user_t (pw_data);
+	  return E_FAILURE;
+	}
+
+      pw_data->use_shadow = 1;
+      pw_data->todo = DO_CREATE_SHADOW;
+      pw_data->sp.sp_namp = pw_data->pw.pw_name;
+      pw_data->sp.sp_lstchg = pw_data->spn.sp_lstchg;
+      pw_data->sp.sp_min = pw_data->spn.sp_min;
+      pw_data->sp.sp_max = pw_data->spn.sp_max;
+      pw_data->sp.sp_warn = pw_data->spn.sp_warn;
+      pw_data->sp.sp_inact = pw_data->spn.sp_inact;
+      pw_data->sp.sp_expire = pw_data->spn.sp_expire;
+    }
+
+  if (write_user_data (pw_data, 0) != 0)
+    {
+      fprintf (stderr, _("Error while changing aging information.\n"));
       free_user_t (pw_data);
       return E_FAILURE;
     }
