@@ -1,4 +1,4 @@
-/* Copyright (C) 2002, 2003 Thorsten Kukuk
+/* Copyright (C) 2002, 2003, 2004 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
@@ -396,7 +396,7 @@ main (int argc, char *argv[])
     }
 
   #ifdef WITH_SELINUX
-  if (is_selinux_enabled ())
+  if (is_selinux_enabled () > 0)
     {
       if ((uid == 0) &&
           (selinux_check_access (pw_data->pw.pw_name, PASSWD__CHSH) != 0))
@@ -405,16 +405,23 @@ main (int argc, char *argv[])
           if (getprevcon (&user_context) < 0)
             user_context =
               (security_context_t) strdup (_("Unknown user context"));
-          syslog (LOG_ALERT,
-                  _("%s is not authorized to change the shell of `%s'."),
-                  user_context, pw_data->pw.pw_name);
-	  fputs (program, stderr);
 	  fprintf (stderr,
-		   _("%s is not authorized to change the shell of `%s'."),
-		   user_context, pw_data->pw.pw_name);
-	  fputs ("\n", stderr);
-	  freecon (user_context);
-	  return E_NOPERM;
+		  _("%s: %s is not authorized to change the shell of `%s'.\n"),
+		   program, user_context, pw_data->pw.pw_name);
+	  if (security_getenforce() > 0)
+	    {
+	      syslog (LOG_ALERT,
+		      _("%s is not authorized to change the shell of `%s'."),
+		      user_context, pw_data->pw.pw_name);
+	      freecon (user_context);
+	      return E_NOPERM;
+	    }
+	  else
+	    {
+	      fprintf (stderr,
+		       _("SELinux is in permissive mode, continuing\n"));
+	      freecon (user_context);
+	    }
         }
     }
 #endif
