@@ -1,4 +1,4 @@
-/* Copyright (C) 2003 Thorsten Kukuk
+/* Copyright (C) 2003, 2004 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
@@ -414,7 +414,7 @@ main (int argc, char **argv)
 	  print_help (program);
 	  return 0;
 	case 'v':
-	  print_version (program, "2003");
+	  print_version (program, "2004");
 	  return 0;
 	case '\254':
 	  print_usage (stdout, program);
@@ -506,7 +506,6 @@ main (int argc, char **argv)
   else
     {
       group_t gr_data;
-      char *cp;
       int retval = E_SUCCESS;
 
       memset (&gr_data, 0, sizeof (gr_data));
@@ -553,7 +552,7 @@ main (int argc, char **argv)
 
 	  if (gr_data.oldclearpwd == NULL)
 	    {
-	      cp = get_ldap_password (gr_data.binddn);
+	      char *cp = get_ldap_password (gr_data.binddn);
 
 	      if (cp)
 		gr_data.oldclearpwd = strdup (cp);
