@@ -52,10 +52,10 @@
 #include <security/pam_appl.h>
 #include <security/pam_misc.h>
 
-#include "getdef.h"
 #include "dbg_log.h"
-#include "rpasswd-client.h"
+#include "logindefs.h"
 #include "error_codes.h"
+#include "rpasswd-client.h"
 
 #ifndef _
 #define _(String) gettext (String)
@@ -545,7 +545,7 @@ cannot handle request version %d; current version is %d"),
       dbg_log ("passwd entry for \"%s\" not found", username);
       /* Dummy authentication. So the user will not see that this
 	 account does not exist.  */
-      ret = pam_start ("passwd", username, &conv, &pamh);
+      ret = pam_start ("rpasswd", username, &conv, &pamh);
       if (ret != PAM_SUCCESS)
 	{
 	  retval = E_PAM_ERROR;
@@ -555,7 +555,7 @@ cannot handle request version %d; current version is %d"),
 	 a failed.  */
       pam_authenticate (pamh, flags);
       pam_end (pamh, PAM_SUCCESS);
-      sleep (getdef_num ("FAIL_DELAY", 1));
+      sleep (getlogindefs_num ("FAIL_DELAY", 1));
       send_string (ssl, ERROR_MSG, "Authentication failure");
       retval = E_FAILURE;
       goto send_finish;
@@ -573,7 +573,7 @@ cannot handle request version %d; current version is %d"),
 	account = username;
 
       /* Do PAM authentification at first.  */
-      ret = pam_start ("passwd", account, &conv, &pamh);
+      ret = pam_start ("rpasswd", account, &conv, &pamh);
       if (ret != PAM_SUCCESS)
 	{
 	  dbg_log ("Couldn't initialize PAM: %s", pam_strerror (pamh, ret));
@@ -597,7 +597,7 @@ cannot handle request version %d; current version is %d"),
 	{
 	  dbg_log (_("User %s: %s"), account,
 		   pam_strerror (pamh, ret));
-	  sleep (getdef_num ("FAIL_DELAY", 1));
+	  sleep (getlogindefs_num ("FAIL_DELAY", 1));
 	  send_string (ssl, ERROR_MSG, pam_strerror (pamh, ret));
 	  retval = E_FAILURE;
 	  goto send_finish;
@@ -644,7 +644,7 @@ cannot handle request version %d; current version is %d"),
 	}
     }
 
-  ret = pam_start ("passwd", username, &conv, &pamh);
+  ret = pam_start ("rpasswd", username, &conv, &pamh);
   if (ret != PAM_SUCCESS)
     {
       dbg_log ("Couldn't initialize PAM: %s", pam_strerror (pamh, ret));
@@ -658,7 +658,7 @@ cannot handle request version %d; current version is %d"),
     {
       dbg_log (_("User %s: %s"), username,
 	       pam_strerror (pamh, ret));
-      sleep (getdef_num ("FAIL_DELAY", 1));
+      sleep (getlogindefs_num ("FAIL_DELAY", 1));
       send_string (ssl, ERROR_MSG, pam_strerror (pamh, ret));
       retval = E_FAILURE;
       goto send_finish;
