@@ -1,4 +1,4 @@
-/* Copyright (C) 2003 Thorsten Kukuk
+/* Copyright (C) 2003, 2004 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
@@ -623,7 +623,6 @@ remove_from_secondary_groups (user_t *pw_data, int have_extrapath,
   } *list = NULL, *item;
   struct group *gr;
   int retval = E_SUCCESS;
-  int asked_for_binddn = 0;
 
   if (have_extrapath)
     {
@@ -686,7 +685,7 @@ remove_from_secondary_groups (user_t *pw_data, int have_extrapath,
 #ifdef USE_LDAP
           if (gr_data->service == S_LDAP)
             {
-              if (pw_data->binddn == NULL && !asked_for_binddn)
+              if (pw_data->binddn == NULL)
                 {
                   pw_data->binddn = get_caller_dn ();
                   if (pw_data->binddn == NULL)
@@ -970,7 +969,7 @@ main (int argc, char **argv)
 	  print_help (program);
 	  return 0;
 	case 'v':
-	  print_version (program, "2003");
+	  print_version (program, "2004");
 	  return 0;
 	case '\254':
 	  print_usage (stdout, program);
